@@ -6,7 +6,6 @@ from unittest.mock import NonCallableMagicMock
 from typing import List, Iterator, Dict, Type, Any, Union
 from abc import ABC
 import logging
-from array import array as Array
 
 # pylint: disable-next=unused-wildcard-import, wildcard-import
 from peakrdl_python.lib import *
@@ -26,7 +25,7 @@ class ReadOnlyRegisterToTest(RegReadOnly):
         __slots__: List[str] = []
 
     # pylint: disable=duplicate-code,too-many-arguments
-    def __init__(self,
+    def __init__(self, *,
                  address: int,
                  accesswidth:int,
                  width:int,
@@ -62,6 +61,13 @@ class ReadOnlyRegisterToTest(RegReadOnly):
         """
         yield self.field
 
+    @property
+    def fields(self) -> Iterator[FieldReadOnly]:
+        """
+        generator that produces has all the readable fields within the register
+        """
+        yield self.field
+
     # build the properties for the fields
     @property
     def field(self) -> FieldToTest:
@@ -83,6 +89,7 @@ class ReadOnlyRegisterToTest(RegReadOnly):
             'field': 'field',
         }
 
+
 class WriteOnlyRegisterToTest(RegWriteOnly):
     """
     Class to represent a register in the register model
@@ -96,7 +103,7 @@ class WriteOnlyRegisterToTest(RegWriteOnly):
         """
         __slots__: List[str] = []
 
-    def __init__(self,
+    def __init__(self, *,
                  address: int,
                  accesswidth: int,
                  width: int,
@@ -132,6 +139,13 @@ class WriteOnlyRegisterToTest(RegWriteOnly):
         """
         yield self.field
 
+    @property
+    def fields(self) -> Iterator[FieldWriteOnly]:
+        """
+        generator that produces has all the readable fields within the register
+        """
+        yield self.field
+
     def write_fields(self, **kwargs: Any) -> None:
         raise NotImplementedError('Not Implemented for the purpose of testing')
 
@@ -156,6 +170,7 @@ class WriteOnlyRegisterToTest(RegWriteOnly):
             'field': 'field',
         }
 
+
 class ReadWriteRegisterToTest(RegReadWrite):
     """
     Class to represent a register in the register model
@@ -169,7 +184,7 @@ class ReadWriteRegisterToTest(RegReadWrite):
         """
         __slots__: List[str] = []
 
-    def __init__(self,
+    def __init__(self, *,
                  address: int,
                  accesswidth: int,
                  width: int,
@@ -205,6 +220,13 @@ class ReadWriteRegisterToTest(RegReadWrite):
         """
         yield self.field
 
+    @property
+    def fields(self) -> Iterator[FieldReadOnly]:
+        """
+        generator that produces has all the readable fields within the register
+        """
+        yield self.field
+
     @property
     def writable_fields(self) -> Iterator[Union['FieldWriteOnly', 'FieldReadWrite']]:
         """
@@ -236,6 +258,7 @@ class ReadWriteRegisterToTest(RegReadWrite):
             'field': 'field',
         }
 
+
 class ReadOnlyRegisterArrayToTest(RegReadOnlyArray):
     """
     Class to represent a register array in the register model
@@ -246,6 +269,7 @@ class ReadOnlyRegisterArrayToTest(RegReadOnlyArray):
     def _element_datatype(self) -> Type[Node]:
         return ReadOnlyRegisterToTest
 
+
 class WriteOnlyRegisterArrayToTest(RegWriteOnlyArray):
     """
     Class to represent a register array in the register model
@@ -256,6 +280,7 @@ class WriteOnlyRegisterArrayToTest(RegWriteOnlyArray):
     def _element_datatype(self) -> Type[Node]:
         return WriteOnlyRegisterToTest
 
+
 class ReadWriteRegisterArrayToTest(RegReadWriteArray):
     """
     Class to represent a register array in the register model
@@ -311,7 +336,7 @@ class CallBackTestWrapper(unittest.TestCase, ABC):
         assert isinstance(data, int)
         self.logger.info(f'write data:{data:X} to address:0x{addr:X}')
 
-    def read_block_addr_space(self, addr: int, width: int, accesswidth: int, length: int) -> Array:
+    def read_block_addr_space(self, addr: int, width: int, accesswidth: int, length: int) -> List:
         """
         Callback to simulate the operation of the package
 
@@ -329,20 +354,10 @@ class CallBackTestWrapper(unittest.TestCase, ABC):
         assert isinstance(accesswidth, int)
         assert isinstance(length, int)
 
-        if width == 32:
-            typecode = 'L'
-        elif width == 64:
-            typecode = 'Q'
-        elif width == 16:
-            typecode = 'I'
-        elif width == 8:
-            typecode = 'B'
-        else:
-            raise ValueError('unhandled memory width')
+        return [0 for x in range(length)]
 
-        return Array(typecode, [0 for x in range(length)])
-
-    def write_block_addr_space(self, addr: int, width: int, accesswidth: int, data: Array) -> None:
+    def write_block_addr_space(self, addr: int,
+                               width: int, accesswidth: int, data: List[int]) -> None:
         """
         Callback to simulate the operation of the package
 
@@ -358,7 +373,7 @@ class CallBackTestWrapper(unittest.TestCase, ABC):
         assert isinstance(addr, int)
         assert isinstance(width, int)
         assert isinstance(accesswidth, int)
-        assert isinstance(data, Array)
+        assert isinstance(data, List)
 
     def setUp(self) -> None:
         # the callbacks need to a faked magic mock so that the methods can be patched in tests
