@@ -62,7 +62,6 @@ class ArrayBase(CallBackTestWrapper, ABC):
         address based on array index
         """
 
-
     def setUp(self) -> None:
 
         class DUTWrapper(AddressMap):
@@ -71,7 +70,7 @@ class ArrayBase(CallBackTestWrapper, ABC):
             """
 
             # pylint: disable=too-many-arguments,duplicate-code
-            def __init__(self,
+            def __init__(self, *,
                          callbacks: Optional[CallbackSet],
                          address: int,
                          logger_handle: str,
@@ -80,18 +79,17 @@ class ArrayBase(CallBackTestWrapper, ABC):
                          dut_dimensions : Tuple[int, ...],
                          RegisterArrayType):
 
-
                 super().__init__(callbacks=callbacks, address=address, logger_handle=logger_handle,
                                  inst_name=inst_name, parent=None )
 
                 self.__dut = RegisterArrayType(logger_handle='dut',
-                                                 inst_name='dut',
-                                                 parent=self,
-                                                 address=address,
-                                                 accesswidth=32,
-                                                 width=32,
-                                                 stride=dut_stride,
-                                                 dimensions=dut_dimensions)
+                                               inst_name='dut',
+                                               parent=self,
+                                               address=address,
+                                               accesswidth=32,
+                                               width=32,
+                                               stride=dut_stride,
+                                               dimensions=dut_dimensions)
 
             def get_memories(self, unroll: bool = False) -> \
                     Iterator[Union[Memory, Tuple[Memory, ...]]]:
@@ -180,10 +178,10 @@ class Test1DArrayReadWrite(ArrayBase):
 
         # try a standard operation
         with patch.object(self.callbacks, 'read_block_callback',
-                          return_value=Array('L', [0 for x in range(10)])) as read_patch, \
+                          return_value=[0 for x in range(10)]) as read_patch, \
                 patch.object(self.callbacks, 'write_block_callback') as write_patch:
 
-            follow_along_array = Array('L', [0 for x in range(10)])
+            follow_along_array = [0 for x in range(10)]
             with self.dut.single_read_modify_write() as dut_context:
                 self.assertEqual(dut_context[2].read(), follow_along_array[2])
                 dut_context[2].write(4)
@@ -204,7 +202,7 @@ class Test1DArrayReadWrite(ArrayBase):
 
         # try with write-back skip
         with patch.object(self.callbacks, 'read_block_callback',
-                          return_value=Array('L', [0 for x in range(10)])) as read_patch, \
+                          return_value=[0 for x in range(10)]) as read_patch, \
                 patch.object(self.callbacks, 'write_block_callback') as write_patch:
             with self.dut.single_read_modify_write(skip_write=True) as dut_context:
                 dut_context[2].write(4)
@@ -221,14 +219,13 @@ class Test1DArrayReadWrite(ArrayBase):
                 with self.dut.single_read_modify_write(verify=True) as dut_context:
                     dut_context[2].write(4)
 
-
-
     def test_blockless_context_manager(self):
         """
         test the context manager that will perform a set of read operation,
         modify write operations with optional read-verify
         """
 
+
 class Test1DArrayReadOnly(ArrayBase):
     """
     Test for 1D arrays
@@ -273,7 +270,7 @@ class Test1DArrayReadOnly(ArrayBase):
 
         # try a standard operation
         with patch.object(self.callbacks, 'read_block_callback',
-                          return_value=Array('L', [x+9 for x in range(10)])) as read_patch, \
+                          return_value=[x+9 for x in range(10)]) as read_patch, \
                 patch.object(self.callbacks, 'write_block_callback') as write_patch:
 
             with self.dut.single_read() as dut_context:
@@ -289,6 +286,7 @@ class Test1DArrayReadOnly(ArrayBase):
         modify write operations with optional read-verify
         """
 
+
 class Test1DArrayWriteOnly(ArrayBase):
     """
     Test for 1D arrays
@@ -336,7 +334,7 @@ class Test1DArrayWriteOnly(ArrayBase):
                           return_value=Array('L', [0 for x in range(10)])) as read_patch, \
                 patch.object(self.callbacks, 'write_block_callback') as write_patch:
 
-            follow_along_array = Array('L', [0 for x in range(10)])
+            follow_along_array = [0 for x in range(10)]
             with self.dut.single_write() as dut_context:
                 for idx, item in enumerate(dut_context):
                     item.write(idx+29)
@@ -354,6 +352,5 @@ class Test1DArrayWriteOnly(ArrayBase):
         """
 
 
-
 if __name__ == '__main__':
     unittest.main()
